<!DOCTYPE html>
<html>
    <head>
        <meta charset="utf-8" />
        <title></title>
        <link rel="stylesheet" href="index.css" />
    </head>
    <body>
        <!-- <ul class="mailmindr-list">
            <li class="mailmindr-list-item">
                <div class="mailmindr-list-item-content">
                    <div class="mailmindr-list-item-buttonpanel">
                        <ul>
                            <li><button>👁</button></li>
                            <li><button>🖋</button></li>
                            <li><button>🗑</button></li>
                        </ul>
                    </div>
                    <div class="mailmindr-list-item-subject">
                        <a href="#">mailmindr - follow-up</a>
                    </div>
                    <div class="mailmindr-list-item-author">
                        Arnd Ißler &lt;hello@mailmindr.net&gt;
                    </div>
                    <div class="mailmindr-list-item-duedate">in 21 days</div>
                    <div class="mailmindr-list-item-note"></div>
                    <div class="mailmindr-list-item-actions">
                        <button class="mailmindr--button-small">Edit</button
                        ><br />
                        <button class="mailmindr--button-small">Remove</button>
                    </div>
                </div>
            </li>
        </ul> -->
        <main>
            <ul id="mailmindr--list" class="mailmindr-list"></ul>
        </main>
        <footer>
            <div>
                <button
                    class="mailmindr-button mailmindr-button--micro mailmindr-sponsorlink mailmindr-button-web"
                    data-web="https://mailmindr.net/sponsor"
                >
                    💜 Become a mailmindr sponsor
                </button>
            </div>
        </footer>
        <script type="module" src="./index.js"></script>
    </body>
</html>
