export const closeMailmindrDialog = async dialogId => {
    window.close();

    // 
    // 
    // 
    // 
    // 
    // 
    // 
    // 
    // 
    // 
    // 

    // 
    // 
    // 
    // 
    // 
};
