@import url('../mailmindr-core.css');
@import url('../../styles/lists.css');

body {
    background-color: var(--in-content-page-background);
    color: var(--in-content-page-color);
    margin: 0;
    padding: 0;
}

fieldset {
    border: unset;
}

fieldset > legend h2 {
    font-size: 17px;
    font-weight: 600;
}

body {
    display: grid;
    grid-template-columns: 240px 1fr;
}

aside {
    background-color: var(--grey-20);
    width: 240px;
    height: 100vh;
    display: flex;
    flex-direction: column;
    position: sticky;
    top: 0;
}

main {
    margin-left: 28px;
    padding-top: 20px;
    max-width: 54vw;
}

input[type='number'] {
    padding: 5px 8px;
    margin: 4px 0;
}

.mailmindr-ui-lock-shim {
    position: absolute;
    left: 0;
    top: 0;
    right: 0;
    bottom: 0;
    width: 100vw;
    height: 100vh;
    background-color: rgba(245, 245, 245, 0.75);

    display: flex;
    flex-direction: column;
    flex-flow: column;
    justify-content: center;
}

.mailmindr-ui-lock-message {
    background-color: white;
    box-shadow: 0px 0px 10px rgba(220, 220, 220, 0.8);
    padding: 1em 2em;
    border-radius: 1em;
    min-width: 40vw;
    max-width: 65vw;
    margin: 0 auto;
    border: 1px solid rgb(220, 220, 220);
}

.mailmindr-options {
    display: grid;
    row-gap: 5px;
    grid-auto-flow: row;
    margin-bottom: 10px;
}

.mailmindr-options--oneline {
    align-items: baseline;
    padding-bottom: 5px;
}

.mailmindr-option-description {
    font-size: 13px;
    font-weight: 400;
    display: block;
    color: var(--grey-50);
}

.mailmindr-list {
    height: 25vh;
    min-height: 20vh;
    min-width: 20vw;
    max-width: 30vw;
}

.mailmindr-list-item {
    font-size: 12px;
}

.mailmindr-list-item:last-child {
    border-bottom: none;
}

.mailmindr-list-item[disabled] {
    color: silver;
}

.mailmindr-options_time-preset-list-button-bar {
    display: flex;
    flex-direction: column;
}

.mailmindr-options_time-preset-list-wrapper {
    display: flex;
    flex-direction: row;
}

.mailmindr-options--oneline select {
    color: inherit !important;
    border: 1px solid transparent;
    border-radius: 2px;
    min-height: 30px;
    font-weight: 400;
    padding: 0 8px;
    text-decoration: none;
    font-size: 1em;
    background-color: rgba(12, 12, 13, 0.1);
    min-width: 20vw;
}

.mailmindr-sidebar-links {
    flex-grow: 1;
}

.mailmindr-sidebar-links ul {
    list-style-type: none;
}

.mailmindr-jump-button {
    text-align: left;
    width: 175px;
}

.mailmindr-jump-button:not(:hover) {
    background-color: unset;
}

.mailmindr-sidebar-footer {
    padding: 0.25em;
    text-align: center;
}
