@import '../styles/photon-colors.css';

* {
    color: var(--grey-90);
    font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, Oxygen,
        Ubuntu, Cantarell, 'Open Sans', 'Helvetica Neue', sans-serif;
}

h1 {
    margin: 0 0 8px;
    font-size: 1.46em;
    font-weight: 300;
    line-height: 1.3em;
}

button {
    appearance: none;
    font-size: 1em;
    margin: 4px 8px;
    height: 24px;
    min-width: 5.5em;
    min-height: 32px;
    color: inherit !important;
    border: 1px solid transparent;
    border-radius: 2px;
}

button:hover {
    border-color: rgba(128, 128, 128, 0.4);
    background-color: rgba(12, 12, 13, 0.2);
}

button[type='submit']:hover {
    /* background-color: rgb(0, 62, 170); */
    background-color: var(--blue-70);
}

button.mailmindr--button-small {
    font-size: 10px;
    height: unset;
    min-height: unset;
}

button.mailmindr-button {
    border-radius: 2px;
    padding-left: 8px;
    padding-right: 8px;
    font-weight: 400;
    min-width: 132px;
    font-size: 13px;
    background-color: var(--grey-90-a10);
}

button.mailmindr-button:hover {
    background-color: var(--grey-90-a20);
}

button.mailmindr-button:disabled {
    opacity: 0.4;
}

button.mailmindr-button--micro {
    height: 24px;
    font-size: 11px;
    min-width: unset;
}

button[type='submit'],
button[type='submit'].mailmindr-button {
    background-color: var(--blue-60);
    color: var(--white-100) !important;
}

button[type='submit']:hover,
button[type='submit'].mailmindr-button:hover {
    background-color: var(--blue-70);
}

button.mailmindr-button--remove {
    color: var(--white-100) !important;
    background-color: var(--red-60);
}

button.mailmindr-button--remove:hover {
    background-color: var(--red-70);
}

select {
    height: 21px;
}
