.mailmindr-list {
    list-style-type: none;
    margin: 0;
    padding: 0;
    max-width: 400px;
}

.mailmindr-list-item {
    border: 1px solid transparent;
    padding: 0.75em;
    margin-bottom: 0.5em;
    min-width: 300px;
}
